@@ -71,7 +71,7 @@ import java.util.concurrent.Exchanger;
  * the construction of the stack will be aborted.
  *
  * @author Bela Ban
- * @version $Id: JChannel.java,v 1.158.2.18 2008/05/28 15:09:38 vlada Exp $
+ * @version $Id: JChannel.java,v 1.158.2.23 2008/11/12 08:08:23 vlada Exp $
  */
 public class JChannel extends Channel {
 
@@ -151,6 +151,7 @@ public class JChannel extends Channel {
 
     protected long sent_msgs=0, received_msgs=0, sent_bytes=0, received_bytes=0;
 
+    private final TP.ProbeHandler probe_handler=new MyProbeHandler();
 
 
 
@@ -589,6 +590,10 @@ public class JChannel extends Channel {
         return retval;
     }
 
+    public Map<String,Object> dumpStats(String protocol_name) {
+        return prot_stack.dumpStats(protocol_name);
+    }
+
     protected Map<String,Long> dumpChannelStats() {
         Map<String,Long> retval=new HashMap<String,Long>();
         retval.put("sent_msgs", new Long(sent_msgs));
@@ -726,6 +731,10 @@ public class JChannel extends Channel {
         return closed ? null : local_addr;
     }
 
+    public String getLocalAddressAsString() {
+        return local_addr != null? local_addr.toString() : "n/a";
+    }
+
 
     /**
      * returns the name of the channel
@@ -1033,7 +1042,7 @@ public class JChannel extends Channel {
             boolean successfulFlush = startFlush(false);
             //http://jira.jboss.com/jira/browse/JGRP-759
             if(!successfulFlush){
-                throw new IllegalStateException("Could not flush the cluster and proceed with state retrieaval");
+                throw new IllegalStateException("Could not flush the cluster and proceed with state retrieval");
             }
         }
 
@@ -1499,8 +1508,9 @@ public class JChannel extends Channel {
     protected final void init(ProtocolStackConfigurator configurator) throws ChannelException {
         if(log.isInfoEnabled())
             log.info("JGroups version: " + Version.description);
-        ConfiguratorFactory.substituteVariables(configurator); // replace vars with system props
+        // ConfiguratorFactory.substituteVariables(configurator); // replace vars with system props
         props=configurator.getProtocolStackString();
+        props=Util.substituteVariable(props);
         prot_stack=new ProtocolStack(this, props);
         try {
             prot_stack.setup(); // Setup protocol stack (creates protocol, calls init() on them)
@@ -1560,6 +1570,9 @@ public class JChannel extends Channel {
         Vector<Address> t=new Vector<Address>(1);
         t.addElement(local_addr);
         my_view=new View(local_addr, 0, t);  // create a dummy view
+
+        TP transport=prot_stack.getTransport();
+        transport.registerProbeHandler(probe_handler);
     }
 
 
@@ -1677,6 +1690,10 @@ public class JChannel extends Channel {
                 if(log.isErrorEnabled())
                     log.error("failed destroying the protocol stack", e);
             }
+
+            TP transport=prot_stack.getTransport();
+            if(transport != null)
+                transport.unregisterProbeHandler(probe_handler);
         }
     }
 
@@ -1847,6 +1864,43 @@ public class JChannel extends Channel {
 
     /* ------------------------------- End of Private Methods ---------------------------------- */
 
+    class MyProbeHandler implements TP.ProbeHandler {
+
+        public Map<String, String> handleProbe(String... keys) {
+            HashMap<String, String> map=new HashMap<String, String>(2);
+            for(String key: keys) {
+                if(key.startsWith("jmx")) {
+                    Map<String, Object> tmp_stats;
+                    int index=key.indexOf("=");
+                    if(index > -1) {
+                        String value=key.substring(index +1);
+                        tmp_stats=dumpStats(value);
+                    }
+                    else
+                        tmp_stats=dumpStats();
+
+                    map.put("jmx", tmp_stats != null? Util.mapToString(tmp_stats) : "null");
+                    continue;
+                }
+                if(key.equals("info")) {
+                    Map<String, Object> tmp_info=getInfo();
+                    map.put("info", tmp_info != null? Util.mapToString(tmp_info) : "null");
+                }
+            }
+
+            map.put("version", Version.description + ", cvs=\"" +  Version.cvs + "\"");
+            if(my_view != null && !map.containsKey("view"))
+                map.put("view", my_view.toString());
+            map.put("local_addr", local_addr != null? local_addr.toString() : "null");
+            map.put("cluster", getClusterName());
+            map.put("member", getLocalAddressAsString() + " (" + getClusterName() + ")");
+            return map;
+        }
+
+        public String[] supportedKeys() {
+            return new String[]{"jmx", "info"};
+        }
+    }
 
     class CloserThread extends Thread {
         final Event evt;
