@@ -1,20 +1,16 @@
-// $Id: RequestCorrelator.java,v 1.40.2.1 2008/01/22 10:00:59 belaban Exp $
+// $Id: RequestCorrelator.java,v 1.40.2.3 2008/11/14 15:16:05 belaban Exp $
 
 package org.jgroups.blocks;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.jgroups.*;
+import org.jgroups.protocols.TP;
 import org.jgroups.stack.Protocol;
-import org.jgroups.util.Scheduler;
-import org.jgroups.util.SchedulerListener;
-import org.jgroups.util.Streamable;
-import org.jgroups.util.Util;
+import org.jgroups.util.*;
 
 import java.io.*;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Stack;
+import java.util.*;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 
@@ -49,7 +45,7 @@ public class RequestCorrelator {
     protected RequestHandler request_handler=null;
 
     /** Possibility for an external marshaller to marshal/unmarshal responses */
-    protected RpcDispatcher.Marshaller marshaller=null;
+    protected RpcDispatcher.Marshaller2 marshaller=null;
 
     /** makes the instance unique (together with IDs) */
     protected String name=null;
@@ -89,6 +85,8 @@ public class RequestCorrelator {
 
     protected boolean started=false;
 
+    private final MyProbeHandler probe_handler=new MyProbeHandler(requests);
+
     protected static final Log log=LogFactory.getLog(RequestCorrelator.class);
 
 
@@ -231,7 +229,12 @@ public class RequestCorrelator {
     }
 
     public void setMarshaller(RpcDispatcher.Marshaller marshaller) {
-        this.marshaller=marshaller;
+        if(marshaller == null)
+            this.marshaller=null;
+        else if(marshaller instanceof RpcDispatcher.Marshaller2)
+            this.marshaller=(RpcDispatcher.Marshaller2)marshaller;
+        else
+            this.marshaller=new RpcDispatcher.MarshallerAdapter(marshaller);
     }
 
     public void sendRequest(long id, List<Address> dest_mbrs, Message msg, RspCollector coll) throws Exception {
@@ -410,6 +413,15 @@ public class RequestCorrelator {
         }
     }
 
+    public void registerProbeHandler(TP transport) {
+        if(transport != null)
+            transport.registerProbeHandler(probe_handler);
+    }
+
+    public void unregisterProbeHandler(TP transport) {
+        if(transport != null)
+            transport.unregisterProbeHandler(probe_handler);
+    }
 
     // .......................................................................
 
@@ -540,8 +552,10 @@ public class RequestCorrelator {
                     Address sender=msg.getSrc();
                     Object retval=null;
                     byte[] buf=msg.getBuffer();
+                    int offset=msg.getOffset(), length=msg.getLength();
                     try {
-                        retval=marshaller != null? marshaller.objectFromByteBuffer(buf) : Util.objectFromByteBuffer(buf);
+                        retval=marshaller != null? marshaller.objectFromByteBuffer(buf, offset, length) :
+                                Util.objectFromByteBuffer(buf, offset, length);
                     }
                     catch(Exception e) {
                         log.error("failed unmarshalling buffer into return value", e);
@@ -576,8 +590,7 @@ public class RequestCorrelator {
      * ID -> <tt>RspCollector</tt>
      */
     private void addEntry(long id, RspCollector coll) {
-        Long id_obj = new Long(id);
-        requests.putIfAbsent(id_obj, coll);
+        requests.putIfAbsent(id, coll);
     }
 
 
@@ -602,9 +615,9 @@ public class RequestCorrelator {
      *
      * @param req the request msg
      */
-    private void handleRequest(Message req, Header hdr) {
+    protected void handleRequest(Message req, Header hdr) {
         Object        retval;
-        byte[]        rsp_buf;
+        Object        rsp_buf; // either byte[] or Buffer
         Header        rsp_hdr;
         Message       rsp;
 
@@ -638,11 +651,11 @@ public class RequestCorrelator {
 
         // changed (bela Feb 20 2004): catch exception and return exception
         try {  // retval could be an exception, or a real value
-            rsp_buf=marshaller != null? marshaller.objectToByteBuffer(retval) : Util.objectToByteBuffer(retval);
+            rsp_buf=marshaller != null? marshaller.objectToBuffer(retval) : Util.objectToByteBuffer(retval);
         }
         catch(Throwable t) {
             try {  // this call should succeed (all exceptions are serializable)
-                rsp_buf=marshaller != null? marshaller.objectToByteBuffer(t) : Util.objectToByteBuffer(t);
+                rsp_buf=marshaller != null? marshaller.objectToBuffer(t) : Util.objectToByteBuffer(t);
             }
             catch(Throwable tt) {
                 if(log.isErrorEnabled()) log.error("failed sending rsp: return value (" + retval + ") is not serializable");
@@ -651,8 +664,10 @@ public class RequestCorrelator {
         }
 
         rsp=req.makeReply();
-        if(rsp_buf != null)
-            rsp.setBuffer(rsp_buf);
+        if(rsp_buf instanceof Buffer)
+            rsp.setBuffer((Buffer)rsp_buf);
+        else if (rsp_buf instanceof byte[])
+            rsp.setBuffer((byte[])rsp_buf);
         rsp_hdr=new Header(Header.RSP, hdr.id, false, name);
         rsp.putHeader(name, rsp_hdr);
         if(log.isTraceEnabled())
@@ -844,6 +859,34 @@ public class RequestCorrelator {
     }
 
 
+    private static class MyProbeHandler implements TP.ProbeHandler {
+        private final ConcurrentMap<Long,RspCollector> requests;
+
+        public MyProbeHandler(ConcurrentMap<Long, RspCollector> requests) {
+            this.requests=requests;
+        }
+
+        public Map<String, String> handleProbe(String... keys) {
+            if(requests == null)
+                return null;
+            Map<String,String> retval=new HashMap<String,String>();
+            for(String key: keys) {
+                if(key.equals("requests")) {
+                    StringBuilder sb=new StringBuilder();
+                    for(Map.Entry<Long,RspCollector> entry: requests.entrySet()) {
+                        sb.append(entry.getKey()).append(": ").append(entry.getValue()).append("\n");
+                    }
+                    retval.put("requests", sb.toString());
+                    break;
+                }
+            }
+            return retval;
+        }
+
+        public String[] supportedKeys() {
+            return new String[]{"requests"};
+        }
+    }
 
 
     /**
