@@ -44,7 +44,7 @@ import java.util.concurrent.locks.ReentrantLock;
  * The {@link #receive(Address, Address, byte[], int, int)} method must
  * be called by subclasses when a unicast or multicast message has been received.
  * @author Bela Ban
- * @version $Id: TP.java,v 1.160.2.29 2008/06/27 07:36:59 belaban Exp $
+ * @version $Id: TP.java,v 1.160.2.35 2008/10/29 12:57:47 vlada Exp $
  */
 public abstract class TP extends Protocol {
 
@@ -145,6 +145,8 @@ public abstract class TP extends Protocol {
 
     /** Number of times init() was called. Incremented on init(), decremented on destroy() */
     int init_count=0;
+    
+    private final ReentrantLock connectLock = new ReentrantLock();
 
 
     /** ================================== OOB thread pool ============================== */
@@ -187,6 +189,8 @@ public abstract class TP extends Protocol {
     
     public void setOOBThreadPoolThreadFactory(ThreadFactory factory) {
         oob_thread_factory=factory;
+        if(oob_thread_pool instanceof ThreadPoolExecutor)
+            ((ThreadPoolExecutor)oob_thread_pool).setThreadFactory(factory);
     }
     
     /** ================================== Regular thread pool ============================== */
@@ -230,6 +234,8 @@ public abstract class TP extends Protocol {
 
     public void setDefaultThreadPoolThreadFactory(ThreadFactory factory) {
         default_thread_factory=factory;
+        if(thread_pool instanceof ThreadPoolExecutor)
+            ((ThreadPoolExecutor)thread_pool).setThreadFactory(factory);
     }
 
     /** ================================== Timer thread pool ================================= */
@@ -273,7 +279,7 @@ public abstract class TP extends Protocol {
 
     /** Maximum number of bytes for messages to be queued until they are sent. This value needs to be smaller
         than the largest datagram packet size in case of UDP */
-    int max_bundle_size=65535;
+    int max_bundle_size=64000;
 
     /** Max number of milliseconds until queued messages are sent. Messages are sent when max_bundle_size or
      * max_bundle_timeout has been exceeded (whichever occurs faster)
@@ -291,7 +297,7 @@ public abstract class TP extends Protocol {
 
     private DiagnosticsHandler diag_handler=null;
     boolean enable_diagnostics=true;
-    String diagnostics_addr="224.0.0.75";
+    String diagnostics_addr="224.0.75.75";
     int    diagnostics_port=7500;
 
     /** If this transport is shared, identifies all the transport instances which are to be shared */
@@ -318,7 +324,7 @@ public abstract class TP extends Protocol {
     long num_msgs_sent=0, num_msgs_received=0, num_bytes_sent=0, num_bytes_received=0;
 
     static  NumberFormat f;
-    private static final int INITIAL_BUFSIZE=1024;
+    private static final int INITIAL_BUFSIZE=4095;
 
     static {
         f=NumberFormat.getNumberInstance();
@@ -348,6 +354,16 @@ public abstract class TP extends Protocol {
         num_oob_msgs_received=num_incoming_msgs_received=0;
     }
 
+    public void registerProbeHandler(ProbeHandler handler) {
+        if(diag_handler != null)
+            diag_handler.registerProbeHandler(handler);
+    }
+
+    public void unregisterProbeHandler(ProbeHandler handler) {
+        if(diag_handler != null)
+            diag_handler.unregisterProbeHandler(handler);
+    }
+
     public long getNumMessagesSent()     {return num_msgs_sent;}
     public long getNumMessagesReceived() {return num_msgs_received;}
     public long getNumBytesSent()        {return num_bytes_sent;}
@@ -1025,11 +1041,6 @@ public abstract class TP extends Protocol {
         str=props.getProperty("max_bundle_size");
         if(str != null) {
             int bundle_size=Integer.parseInt(str);
-            if(bundle_size > max_bundle_size) {
-                if(log.isErrorEnabled()) log.error("max_bundle_size (" + bundle_size +
-                        ") is greater than largest TP fragmentation size (" + max_bundle_size + ')');
-                return false;
-            }
             if(bundle_size <= 0) {
                 if(log.isErrorEnabled()) log.error("max_bundle_size (" + bundle_size + ") is <= 0");
                 return false;
@@ -1572,19 +1583,29 @@ public abstract class TP extends Protocol {
             case Event.CONNECT_WITH_STATE_TRANSFER:
                 channel_name=(String)evt.getArg();
                 header=new TpHeader(channel_name);
-            setInAllThreadFactories(channel_name, local_addr, thread_naming_pattern);
+                setInAllThreadFactories(channel_name, local_addr, thread_naming_pattern);
                 setThreadNames();
+                connectLock.lock();
                 try {
                     handleConnect();
                 }
                 catch(Exception e) {
                     throw new RuntimeException(e);
                 }
+                finally {
+                    connectLock.unlock();
+                }
                 return null;
 
             case Event.DISCONNECT:
                 unsetThreadNames();
-                handleDisconnect();
+                connectLock.lock();
+                try {
+                    handleDisconnect();
+                }
+                finally {
+                    connectLock.unlock();
+                }             
                 break;
 
             case Event.CONFIG:
@@ -2069,11 +2090,24 @@ public abstract class TP extends Protocol {
         }
     }
 
+    public interface ProbeHandler {
+        /**
+         * Handles a probe. For each key that is handled, the key and its result should be in the returned map.
+         * @param keys
+         * @return Map<String,String>. A map of keys and values. A null return value is permissible.
+         */
+        Map<String,String> handleProbe(String... keys);
+
+        /** Returns a list of supported keys */
+        String[] supportedKeys();
+    }
+
 
     private class DiagnosticsHandler implements Runnable {
     	public static final String THREAD_NAME = "DiagnosticsHandler"; 
-        Thread thread=null;
-        MulticastSocket diag_sock=null;
+        private Thread thread=null;
+        private MulticastSocket diag_sock=null;
+        private final Set<ProbeHandler> handlers=new HashSet<ProbeHandler>();
 
         DiagnosticsHandler() {
         }
@@ -2082,7 +2116,47 @@ public abstract class TP extends Protocol {
         	return thread;
         }
 
+        void registerProbeHandler(ProbeHandler handler) {
+            if(handler != null)
+                handlers.add(handler);
+        }
+
+        void unregisterProbeHandler(ProbeHandler handler) {
+            if(handler != null)
+                handlers.remove(handler);
+        }
+
         void start() throws IOException {
+
+            registerProbeHandler(new ProbeHandler() {
+
+                public Map<String, String> handleProbe(String... keys) {
+                    Map<String,String> retval=new HashMap<String,String>(2);
+                    for(String key: keys) {
+                        if(key.equals("dump")) {
+                            retval.put("dump", Util.dumpThreads());
+                            continue;
+                        }
+                        if(key.equals("keys")) {
+                            StringBuilder sb=new StringBuilder();
+                            for(ProbeHandler handler: handlers) {
+                                String[] tmp=handler.supportedKeys();
+                                if(tmp != null && tmp.length > 0) {
+                                    for(String s: tmp)
+                                        sb.append(s).append(" ");
+                                }
+                            }
+                            retval.put("keys", sb.toString());
+                        }
+                    }
+                    return retval;
+                }
+
+                public String[] supportedKeys() {
+                    return new String[]{"dump", "keys"};
+                }
+            });
+
             diag_sock=new MulticastSocket(diagnostics_port);
             java.util.List interfaces=Util.getAllAvailableInterfaces();
             bindToInterfaces(interfaces, diag_sock);
@@ -2097,6 +2171,7 @@ public abstract class TP extends Protocol {
         void stop() {
             if(diag_sock != null)
                 diag_sock.close();
+            handlers.clear();
             if(thread != null){
                 try{
                     thread.join(Global.THREAD_SHUTDOWN_WAIT_TIME);
@@ -2122,6 +2197,52 @@ public abstract class TP extends Protocol {
             }
         }
 
+        private void handleDiagnosticProbe(SocketAddress sender, DatagramSocket sock, String request) {
+            StringTokenizer tok=new StringTokenizer(request);
+            List<String> list=new ArrayList<String>(10);
+
+            while(tok.hasMoreTokens()) {
+                String req=tok.nextToken().trim();
+                if(req.length() > 0)
+                    list.add(req);
+            }
+
+            String[] tokens=new String[list.size()];
+            for(int i=0; i < list.size(); i++)
+                tokens[i]=list.get(i);
+
+
+            for(ProbeHandler handler: handlers) {
+                Map<String, String> map=handler.handleProbe(tokens);
+                if(map == null || map.isEmpty())
+                    continue;
+                if(!map.containsKey("local_addr"))
+                    map.put("local_addr", local_addr != null? local_addr.toString() : "n/a");
+                if(!map.containsKey("cluster"))
+                    map.put("cluster", channel_name != null? channel_name : "n/a");
+                StringBuilder info=new StringBuilder();
+                for(Map.Entry<String,String> entry: map.entrySet()) {
+                    info.append(entry.getKey()).append("=").append(entry.getValue()).append("\r\n");
+                }
+
+                byte[] diag_rsp=info.toString().getBytes();
+                if(log.isDebugEnabled())
+                    log.debug("sending diag response to " + sender);
+                try {
+                    sendResponse(sock, sender, diag_rsp);
+                }
+                catch(Throwable t) {
+                    if(log.isErrorEnabled())
+                        log.error("failed sending diag rsp to " + sender, t);
+                }
+            }
+        }
+
+        private void sendResponse(DatagramSocket sock, SocketAddress sender, byte[] buf) throws IOException {
+            DatagramPacket p=new DatagramPacket(buf, 0, buf.length, sender);
+            sock.send(p);
+        }
+
         private void bindToInterfaces(java.util.List<NetworkInterface> interfaces, MulticastSocket s) {
             SocketAddress group_addr=new InetSocketAddress(diagnostics_addr, diagnostics_port);
             for(Iterator<NetworkInterface> it=interfaces.iterator(); it.hasNext();) {
