@@ -218,14 +218,18 @@ public class FLUSH extends Protocol {
             }
         }
 
-        if(!successfulFlush && numberOfAttempts > 0){                  
-            waitForFlushCompletion(retry_timeout);      
-            if(log.isDebugEnabled()){               
-                log.debug("Retrying FLUSH at " + localAddress                                                  
-                          + ". Attempts left "
-                          + numberOfAttempts);
+        if(!successfulFlush && numberOfAttempts > 0) {
+            waitForFlushCompletion(retry_timeout);
+            Boolean result=flush_promise.getResult(100);
+            successfulFlush = result !=null && result.booleanValue();
+            if(!successfulFlush){
+                if(log.isDebugEnabled()) {
+                    log.debug("Retrying FLUSH at " + localAddress
+                              + ". Attempts left "
+                              + numberOfAttempts);
+                }
+                successfulFlush=startFlush(flushParticipants, --numberOfAttempts, true);
             }
-            successfulFlush = startFlush(flushParticipants, --numberOfAttempts, true);
         }
         return successfulFlush;
     }
@@ -332,7 +336,7 @@ public class FLUSH extends Protocol {
                         return up_prot.up(evt);                     
                     case FlushHeader.START_FLUSH:
                         Collection<Address> fp=fh.flushParticipants;
-                        boolean amIParticipant = fp != null && fp.contains(localAddress);
+                        boolean amIParticipant = (fp != null && fp.contains(localAddress)) || msg.getSrc().equals(localAddress);
                         if(amIParticipant){
                             handleStartFlush(msg, fh);
                         }         
@@ -351,9 +355,9 @@ public class FLUSH extends Protocol {
                     case FlushHeader.STOP_FLUSH:
                         onStopFlush();
                         break;
-                    case FlushHeader.ABORT_FLUSH:
-                        synchronized(sharedLock){
-                            flushCompletedMap.clear();
+                    case FlushHeader.ABORT_FLUSH: 
+                        synchronized(sharedLock){                            
+                            flushCompletedMap.remove(msg.getSrc());
                         }
                         flush_promise.setResult(Boolean.FALSE);
                         break;                               
@@ -521,17 +525,18 @@ public class FLUSH extends Protocol {
         }
         if(proceed){          
             onStartFlush(flushRequester, fh);
-        } else{  
-            if(log.isDebugEnabled()){
-                log.debug("Rejecting flush at " + localAddress
-                          + " to flush requester "
-                          + abortFlushCoordinator
-                          + " coordinator is "
-                          + proceedFlushCoordinator);
-            }                        
-            rejectFlush(fh.viewID, abortFlushCoordinator);    
-            if(proceedFlushCoordinator != null)
+        } else{                                     
+            if(proceedFlushCoordinator != null){
+                if(log.isDebugEnabled()){
+                    log.debug("Rejecting flush at " + localAddress
+                              + " to flush requester "
+                              + abortFlushCoordinator
+                              + " coordinator is "
+                              + proceedFlushCoordinator);
+                }         
+                rejectFlush(fh.viewID, abortFlushCoordinator);
                 onStartFlush(proceedFlushCoordinator, fh);
+            }
         }
     }
 
@@ -675,39 +680,45 @@ public class FLUSH extends Protocol {
         if(stats){
             startFlushTime = System.currentTimeMillis();
             numberOfFlushes += 1;
-        }        
+        }
+        boolean proceed = false;
         synchronized(sharedLock){
             flushCoordinator = flushStarter;
             flushMembers.clear();
             if(fh.flushParticipants != null){
                 flushMembers.addAll(fh.flushParticipants);
             }
-            flushMembers.removeAll(suspected);            
+            proceed = flushMembers.contains(localAddress);
+            flushMembers.removeAll(suspected);           
         }
         
-        if(sentBlock.compareAndSet(false, true)){
-            //ensures that we do not repeat block event
-            //and that we do not send block event to non participants            
-            sendBlockUpToChannel();            
-            synchronized(blockMutex){
-                isBlockingFlushDown = true;
+        if(proceed) {
+            if(sentBlock.compareAndSet(false, true)) {
+                //ensures that we do not repeat block event
+                //and that we do not send block event to non participants            
+                sendBlockUpToChannel();
+                synchronized(blockMutex) {
+                    isBlockingFlushDown=true;
+                }
             }
-        }
-        else{
+            else {
+                if(log.isDebugEnabled())
+                    log.debug("Received START_FLUSH at " + localAddress
+                              + " but not sending BLOCK up");
+            }
+
+            Digest digest=(Digest)down_prot.down(new Event(Event.GET_DIGEST));
+            FlushHeader fhr=new FlushHeader(FlushHeader.FLUSH_COMPLETED, fh.viewID);
+            fhr.addDigest(digest);
+
+            Message msg=new Message(flushStarter);
+            msg.putHeader(getName(), fhr);
+            down_prot.down(new Event(Event.MSG, msg));
             if(log.isDebugEnabled())
-                log.debug("Received START_FLUSH at " + localAddress + " but not sending BLOCK up");
-        }                        
-            
-        Digest digest = (Digest) down_prot.down(new Event(Event.GET_DIGEST));
-        FlushHeader fhr = new FlushHeader(FlushHeader.FLUSH_COMPLETED, fh.viewID);
-        fhr.addDigest(digest);            
-        
-        Message msg = new Message(flushStarter);
-        msg.putHeader(getName(), fhr);
-        down_prot.down(new Event(Event.MSG, msg));
-        if(log.isDebugEnabled())
-            log.debug("Received START_FLUSH at " + localAddress
-                      + " responded with FLUSH_COMPLETED to " + flushStarter );
+                log.debug("Received START_FLUSH at " + localAddress
+                          + " responded with FLUSH_COMPLETED to "
+                          + flushStarter);
+        }
               
     }
     
@@ -716,7 +727,7 @@ public class FLUSH extends Protocol {
         boolean needsReconciliationPhase = false;
         synchronized(sharedLock){
             flushCompletedMap.put(address, digest);
-            flushCompleted=flushCompletedMap.size() >= flushMembers.size() && flushCompletedMap.keySet().containsAll(flushMembers);           
+            flushCompleted=flushCompletedMap.size() >=  flushMembers.size() && !flushMembers.isEmpty() && flushCompletedMap.keySet().containsAll(flushMembers);           
 
             if(log.isDebugEnabled())
                 log.debug("At " + localAddress
@@ -724,6 +735,8 @@ public class FLUSH extends Protocol {
                           + address
                           + ",completed "
                           + flushCompleted
+                          + ",flushMembers "
+                          + flushMembers
                           + ",flushCompleted "
                           + flushCompletedMap.keySet());
 
@@ -741,8 +754,8 @@ public class FLUSH extends Protocol {
                 fh.addDigest(d);
                 msg.putHeader(getName(), fh);
 
-                if(log.isTraceEnabled())
-                    log.trace("Reconciling flush mebers due to virtual synchrony gap, digest is " + d
+                if(log.isDebugEnabled())
+                    log.debug("At "+ localAddress + " reconciling flush mebers due to virtual synchrony gap, digest is " + d
                               + " flush members are "
                               + flushMembers);
 
