@@ -1,12 +1,14 @@
 package org.jgroups.util;
 
 import org.apache.commons.logging.LogFactory;
+import org.apache.commons.logging.Log;
 import org.jgroups.*;
 import org.jgroups.auth.AuthToken;
 import org.jgroups.conf.ClassConfigurator;
 import org.jgroups.protocols.FD;
 import org.jgroups.protocols.PingHeader;
 import org.jgroups.protocols.UdpHeader;
+import org.jgroups.protocols.PingRsp;
 import org.jgroups.protocols.pbcast.NakAckHeader;
 import org.jgroups.stack.IpAddress;
 import org.jgroups.stack.Protocol;
@@ -28,7 +30,7 @@ import java.util.*;
 /**
  * Collection of various utility routines that can not be assigned to other classes.
  * @author Bela Ban
- * @version $Id: Util.java,v 1.137.2.4 2008/05/21 09:52:34 belaban Exp $
+ * @version $Id: Util.java,v 1.137.2.8 2008/10/10 10:07:59 belaban Exp $
  */
 public class Util {
 
@@ -926,7 +928,7 @@ public class Util {
 
     /** Returns a random value in the range [1 - range] */
     public static long random(long range) {
-        return (long)((Math.random() * 100000) % range) + 1;
+        return (long)((Math.random() * range) % range) + 1;
     }
 
 
@@ -1140,7 +1142,23 @@ public class Util {
         catch(Exception e) {  // it is not an object
             return "";
         }
+    }
 
+    public static String mapToString(Map<? extends Object,? extends Object> map) {
+        if(map == null)
+            return "null";
+        StringBuilder sb=new StringBuilder();
+        for(Map.Entry<? extends Object,? extends Object> entry: map.entrySet()) {
+            Object key=entry.getKey();
+            Object val=entry.getValue();
+            sb.append(key).append("=");
+            if(val == null)
+                sb.append("null");
+            else
+                sb.append(val);
+            sb.append("\n");
+    }
+        return sb.toString();
     }
 
 
@@ -1553,6 +1571,24 @@ public class Util {
     }
 
 
+    public static String printPingRsps(List<PingRsp> rsps) {
+        StringBuilder sb=new StringBuilder();
+        if(rsps != null) {
+            int total=rsps.size();
+            int servers=0, clients=0, coords=0;
+            for(PingRsp rsp: rsps) {
+                if(rsp.isCoord())
+                    coords++;
+                if(rsp.isServer())
+                    servers++;
+                else
+                    clients++;
+            }
+            sb.append(total + " total (" + servers + " servers (" + coords + " coord), " + clients + " clients)");
+        }
+        return sb.toString();
+    }
+
     /**
      Makes sure that we detect when a peer connection is in the closed state (not closed while we send data,
      but before we send data). Two writes ensure that, if the peer closed the connection, the first write
@@ -2034,6 +2070,42 @@ public class Util {
     }
 
 
+    public static MulticastSocket createMulticastSocket(int port) throws IOException {
+        return createMulticastSocket(null, port, null);
+    }
+
+    public static MulticastSocket createMulticastSocket(InetAddress mcast_addr, int port, Log log) throws IOException {
+        if(mcast_addr != null && !mcast_addr.isMulticastAddress()) {
+            if(log != null && log.isWarnEnabled())
+                log.warn("mcast_addr (" + mcast_addr + ") is not a multicast address, will be ignored");
+            return new MulticastSocket(port);
+        }
+
+        SocketAddress saddr=new InetSocketAddress(mcast_addr, port);
+        MulticastSocket retval=null;
+
+        try {
+            retval=new MulticastSocket(saddr);
+        }
+        catch(IOException ex) {
+            if(log != null && log.isWarnEnabled()) {
+                StringBuilder sb=new StringBuilder();
+                String type=mcast_addr != null ? mcast_addr instanceof Inet4Address? "IPv4" : "IPv6" : "n/a";
+                sb.append("could not bind to " + mcast_addr + " (" + type + " address)");
+                sb.append("; make sure your mcast_addr is of the same type as the IP stack (IPv4 or IPv6).");
+                sb.append("\nWill ignore mcast_addr, but this may lead to cross talking " +
+                        "(see http://www.jboss.com/wiki/Edit.jsp?page=CrossTalking for details). ");
+                sb.append("\nException was: " + ex);
+                log.warn(sb);
+            }
+        }
+        if(retval == null)
+            retval=new MulticastSocket(port);
+        return retval;
+    }
+
+
+
     /**
      * Returns the address of the interface to use defined by bind_addr and bind_interface
      * @param props
@@ -2092,18 +2164,25 @@ public class Util {
 
 
     public static boolean checkForLinux() {
-        String os=System.getProperty("os.name");
-        return os != null && os.toLowerCase().startsWith("linux");
+        return checkForPresence("os.name", "linux");
     }
 
     public static boolean checkForSolaris() {
-        String os=System.getProperty("os.name");
-        return os != null && os.toLowerCase().startsWith("sun");
+        return checkForPresence("os.name", "sun");
     }
 
     public static boolean checkForWindows() {
-        String os=System.getProperty("os.name");
-        return os != null && os.toLowerCase().startsWith("win");
+        return checkForPresence("os.name", "win");
+    }
+
+    private static boolean checkForPresence(String key, String value) {
+        try {
+            String tmp=System.getProperty(key);
+            return tmp != null && tmp.trim().toLowerCase().startsWith(value);
+        }
+        catch(Throwable t) {
+            return false;
+        }
     }
 
     public static void prompt(String s) {
